@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,14 +20,41 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-__version__ = '1.1.2'  # Remember to change in setup.py as well!
-__author__ = 'Raphaël Barrois <raphael.barrois@polytechnique.org>'
+__version__ = '2.0.2'
+__author__ = 'Raphaël Barrois <raphael.barrois+fboy@polytechnique.org>'
 
-from base import (
+from .base import (
     Factory,
+    BaseDictFactory,
+    DictFactory,
+    BaseListFactory,
+    ListFactory,
+    MogoFactory,
     StubFactory,
     DjangoModelFactory,
 
+    BUILD_STRATEGY,
+    CREATE_STRATEGY,
+    STUB_STRATEGY,
+    use_strategy,
+)
+
+from .declarations import (
+    LazyAttribute,
+    Iterator,
+    Sequence,
+    LazyAttributeSequence,
+    SelfAttribute,
+    ContainerAttribute,
+    SubFactory,
+    Dict,
+    List,
+    PostGeneration,
+    PostGenerationMethodCall,
+    RelatedFactory,
+)
+
+from .helpers import (
     build,
     create,
     stub,
@@ -41,30 +68,11 @@ from base import (
     generate_batch,
     simple_generate_batch,
 
-    BUILD_STRATEGY,
-    CREATE_STRATEGY,
-    STUB_STRATEGY,
-
-    DJANGO_CREATION,
-    NAIVE_BUILD,
-    MOGO_BUILD,
-)
-
-from declarations import (
-    LazyAttribute,
-    Iterator,
-    InfiniteIterator,
-    Sequence,
-    LazyAttributeSequence,
-    SelfAttribute,
-    ContainerAttribute,
-    SubFactory,
-
     lazy_attribute,
     iterator,
-    infinite_iterator,
     sequence,
     lazy_attribute_sequence,
     container_attribute,
+    post_generation,
 )
 
