@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,12 +20,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import unittest
 import warnings
 
 from factory import base
 from factory import declarations
 
+from .compat import unittest
+
 class TestObject(object):
     def __init__(self, one=None, two=None, three=None, four=None):
         self.one = one
@@ -34,48 +35,77 @@ class TestObject(object):
         self.four = four
 
 class FakeDjangoModel(object):
-    class FakeDjangoManager(object):
-        def create(self, **kwargs):
-            fake_model = FakeDjangoModel(**kwargs)
-            fake_model.id = 1
-            return fake_model
-
-    objects = FakeDjangoManager()
+    @classmethod
+    def create(cls, **kwargs):
+        instance = cls(**kwargs)
+        instance.id = 1
+        return instance
 
     def __init__(self, **kwargs):
-        for name, value in kwargs.iteritems():
+        for name, value in kwargs.items():
             setattr(self, name, value)
             self.id = None
 
+class FakeModelFactory(base.Factory):
+    ABSTRACT_FACTORY = True
+
+    @classmethod
+    def _create(cls, target_class, *args, **kwargs):
+        return target_class.create(**kwargs)
+
+
 class TestModel(FakeDjangoModel):
     pass
 
 
 class SafetyTestCase(unittest.TestCase):
-    def testBaseFactory(self):
-        self.assertRaises(RuntimeError, base.BaseFactory)
+    def test_base_factory(self):
+        self.assertRaises(base.FactoryError, base.BaseFactory)
+
+
+class AbstractFactoryTestCase(unittest.TestCase):
+    def test_factory_for_optional(self):
+        """Ensure that FACTORY_FOR is optional for ABSTRACT_FACTORY."""
+        class TestObjectFactory(base.Factory):
+            ABSTRACT_FACTORY = True
+
+        # Passed
 
 
 class FactoryTestCase(unittest.TestCase):
-    def testDisplay(self):
+    def test_factory_for(self):
+        class TestObjectFactory(base.Factory):
+            FACTORY_FOR = TestObject
+
+        self.assertEqual(TestObject, TestObjectFactory.FACTORY_FOR)
+        obj = TestObjectFactory.build()
+        self.assertFalse(hasattr(obj, 'FACTORY_FOR'))
+
+    def test_display(self):
         class TestObjectFactory(base.Factory):
             FACTORY_FOR = FakeDjangoModel
 
         self.assertIn('TestObjectFactory', str(TestObjectFactory))
         self.assertIn('FakeDjangoModel', str(TestObjectFactory))
 
-    def testLazyAttributeNonExistentParam(self):
+    def test_lazy_attribute_non_existent_param(self):
         class TestObjectFactory(base.Factory):
+            FACTORY_FOR = TestObject
+
             one = declarations.LazyAttribute(lambda a: a.does_not_exist )
 
         self.assertRaises(AttributeError, TestObjectFactory)
 
-    def testInheritanceWithSequence(self):
+    def test_inheritance_with_sequence(self):
         """Tests that sequence IDs are shared between parent and son."""
         class TestObjectFactory(base.Factory):
+            FACTORY_FOR = TestObject
+
             one = declarations.Sequence(lambda a: a)
 
         class TestSubFactory(TestObjectFactory):
+            FACTORY_FOR = TestObject
+
             pass
 
         parent = TestObjectFactory.build()
@@ -85,107 +115,111 @@ class FactoryTestCase(unittest.TestCase):
         ones = set([x.one for x in (parent, alt_parent, sub, alt_sub)])
         self.assertEqual(4, len(ones))
 
+
 class FactoryDefaultStrategyTestCase(unittest.TestCase):
     def setUp(self):
-        self.default_strategy = base.Factory.default_strategy
+        self.default_strategy = base.Factory.FACTORY_STRATEGY
 
     def tearDown(self):
-        base.Factory.default_strategy = self.default_strategy
+        base.Factory.FACTORY_STRATEGY = self.default_strategy
 
-    def testBuildStrategy(self):
-        base.Factory.default_strategy = base.BUILD_STRATEGY
+    def test_build_strategy(self):
+        base.Factory.FACTORY_STRATEGY = base.BUILD_STRATEGY
 
         class TestModelFactory(base.Factory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         test_model = TestModelFactory()
         self.assertEqual(test_model.one, 'one')
         self.assertFalse(test_model.id)
 
-    def testCreateStrategy(self):
-        # Default default_strategy
+    def test_create_strategy(self):
+        # Default FACTORY_STRATEGY
+
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
 
-        class TestModelFactory(base.Factory):
             one = 'one'
 
         test_model = TestModelFactory()
         self.assertEqual(test_model.one, 'one')
         self.assertTrue(test_model.id)
 
-    def testStubStrategy(self):
-        base.Factory.default_strategy = base.STUB_STRATEGY
+    def test_stub_strategy(self):
+        base.Factory.FACTORY_STRATEGY = base.STUB_STRATEGY
 
         class TestModelFactory(base.Factory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         test_model = TestModelFactory()
         self.assertEqual(test_model.one, 'one')
         self.assertFalse(hasattr(test_model, 'id'))  # We should have a plain old object
 
-    def testUnknownStrategy(self):
-        base.Factory.default_strategy = 'unknown'
+    def test_unknown_strategy(self):
+        base.Factory.FACTORY_STRATEGY = 'unknown'
 
         class TestModelFactory(base.Factory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         self.assertRaises(base.Factory.UnknownStrategy, TestModelFactory)
 
-    def testStubWithNonStubStrategy(self):
+    def test_stub_with_non_stub_strategy(self):
         class TestModelFactory(base.StubFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
-        TestModelFactory.default_strategy = base.CREATE_STRATEGY
+        TestModelFactory.FACTORY_STRATEGY = base.CREATE_STRATEGY
 
         self.assertRaises(base.StubFactory.UnsupportedStrategy, TestModelFactory)
 
-        TestModelFactory.default_strategy = base.BUILD_STRATEGY
+        TestModelFactory.FACTORY_STRATEGY = base.BUILD_STRATEGY
         self.assertRaises(base.StubFactory.UnsupportedStrategy, TestModelFactory)
 
+    def test_change_strategy(self):
+        @base.use_strategy(base.CREATE_STRATEGY)
+        class TestModelFactory(base.StubFactory):
+            FACTORY_FOR = TestModel
+
+            one = 'one'
+
+        self.assertEqual(base.CREATE_STRATEGY, TestModelFactory.FACTORY_STRATEGY)
+
+
 class FactoryCreationTestCase(unittest.TestCase):
-    def testFactoryFor(self):
+    def test_factory_for(self):
         class TestFactory(base.Factory):
             FACTORY_FOR = TestObject
 
         self.assertTrue(isinstance(TestFactory.build(), TestObject))
 
-    def testAutomaticAssociatedClassDiscovery(self):
-        class TestObjectFactory(base.Factory):
-            pass
-
-        self.assertTrue(isinstance(TestObjectFactory.build(), TestObject))
-
-    def testDeprecationWarning(self):
-        """Make sure the 'auto-discovery' deprecation warning is issued."""
-
-        with warnings.catch_warnings(record=True) as w:
-            # Clear the warning registry.
-            if hasattr(base, '__warningregistry__'):
-                base.__warningregistry__.clear()
-
-            warnings.simplefilter('always')
-            class TestObjectFactory(base.Factory):
-                pass
-
-            self.assertEqual(1, len(w))
-            self.assertIn('deprecated', str(w[0].message))
-
-    def testStub(self):
+    def test_stub(self):
         class TestFactory(base.StubFactory):
             pass
 
-        self.assertEqual(TestFactory.default_strategy, base.STUB_STRATEGY)
+        self.assertEqual(TestFactory.FACTORY_STRATEGY, base.STUB_STRATEGY)
 
-    def testInheritanceWithStub(self):
+    def test_inheritance_with_stub(self):
         class TestObjectFactory(base.StubFactory):
+            FACTORY_FOR = TestObject
+
             pass
 
         class TestFactory(TestObjectFactory):
             pass
 
-        self.assertEqual(TestFactory.default_strategy, base.STUB_STRATEGY)
+        self.assertEqual(TestFactory.FACTORY_STRATEGY, base.STUB_STRATEGY)
+
+    def test_custom_creation(self):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
 
-    def testCustomCreation(self):
-        class TestModelFactory(base.Factory):
             @classmethod
             def _prepare(cls, create, **kwargs):
                 kwargs['four'] = 4
@@ -203,15 +237,7 @@ class FactoryCreationTestCase(unittest.TestCase):
 
     # Errors
 
-    def testNoAssociatedClassWithAutodiscovery(self):
-        try:
-            class TestFactory(base.Factory):
-                pass
-            self.fail()
-        except base.Factory.AssociatedClassError as e:
-            self.assertTrue('autodiscovery' in str(e))
-
-    def testNoAssociatedClassWithoutAutodiscovery(self):
+    def test_no_associated_class(self):
         try:
             class Test(base.Factory):
                 pass
@@ -220,5 +246,27 @@ class FactoryCreationTestCase(unittest.TestCase):
             self.assertTrue('autodiscovery' not in str(e))
 
 
+class PostGenerationParsingTestCase(unittest.TestCase):
+
+    def test_extraction(self):
+        class TestObjectFactory(base.Factory):
+            FACTORY_FOR = TestObject
+
+            foo = declarations.PostGenerationDeclaration()
+
+        self.assertIn('foo', TestObjectFactory._postgen_declarations)
+
+    def test_classlevel_extraction(self):
+        class TestObjectFactory(base.Factory):
+            FACTORY_FOR = TestObject
+
+            foo = declarations.PostGenerationDeclaration()
+            foo__bar = 42
+
+        self.assertIn('foo', TestObjectFactory._postgen_declarations)
+        self.assertIn('foo__bar', TestObjectFactory._declarations)
+
+
+
 if __name__ == '__main__':
     unittest.main()
