@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,12 +20,12 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import unittest
-
 from factory import base
 from factory import containers
 from factory import declarations
 
+from .compat import unittest
+
 class LazyStubTestCase(unittest.TestCase):
     def test_basic(self):
         stub = containers.LazyStub({'one': 1, 'two': 2})
@@ -178,6 +178,17 @@ class DeclarationDictTestCase(unittest.TestCase):
         self.assertEqual(set(['one', 'three']), set(d))
         self.assertEqual(set([1, 3]), set(d.values()))
 
+    def test_update_with_public_ignores_factory_attributes(self):
+        """Ensure that a DeclarationDict ignores FACTORY_ keys."""
+        d = containers.DeclarationDict()
+        d.update_with_public({
+            'one': 1,
+            'FACTORY_FOR': 2,
+            'FACTORY_ARG_PARAMETERS': 3,
+        })
+        self.assertEqual(['one'], list(d))
+        self.assertEqual([1], list(d.values()))
+
 
 class AttributeBuilderTestCase(unittest.TestCase):
     def test_empty(self):
@@ -242,7 +253,7 @@ class AttributeBuilderTestCase(unittest.TestCase):
         self.assertEqual({'one': 2}, ab.build(create=False))
 
     def test_factory_defined_sequence(self):
-        seq = declarations.Sequence(lambda n: 'xx' + n)
+        seq = declarations.Sequence(lambda n: 'xx%d' % n)
 
         class FakeFactory(object):
             @classmethod
@@ -259,7 +270,7 @@ class AttributeBuilderTestCase(unittest.TestCase):
         self.assertEqual({'one': 'xx1'}, ab.build(create=False))
 
     def test_additionnal_sequence(self):
-        seq = declarations.Sequence(lambda n: 'xx' + n)
+        seq = declarations.Sequence(lambda n: 'xx%d' % n)
 
         class FakeFactory(object):
             @classmethod
@@ -276,8 +287,8 @@ class AttributeBuilderTestCase(unittest.TestCase):
         self.assertEqual({'one': 1, 'two': 'xx1'}, ab.build(create=False))
 
     def test_replaced_sequence(self):
-        seq = declarations.Sequence(lambda n: 'xx' + n)
-        seq2 = declarations.Sequence(lambda n: 'yy' + n)
+        seq = declarations.Sequence(lambda n: 'xx%d' % n)
+        seq2 = declarations.Sequence(lambda n: 'yy%d' % n)
 
         class FakeFactory(object):
             @classmethod
@@ -316,6 +327,24 @@ class AttributeBuilderTestCase(unittest.TestCase):
         ab = containers.AttributeBuilder(FakeFactory, {'one': 4, 'three': la})
         self.assertEqual({'one': 4, 'two': 8, 'three': 8}, ab.build(create=False))
 
+    def test_subfields(self):
+        class FakeInnerFactory(object):
+            pass
+
+        sf = declarations.SubFactory(FakeInnerFactory)
+
+        class FakeFactory(object):
+            @classmethod
+            def declarations(cls, extra):
+                d = {'one': sf, 'two': 2}
+                d.update(extra)
+                return d
+
+        ab = containers.AttributeBuilder(FakeFactory, {'one__blah': 1, 'two__bar': 2})
+        self.assertTrue(ab.has_subfields(sf))
+        self.assertEqual(['one'], list(ab._subfields.keys()))
+        self.assertEqual(2, ab._attrs['two__bar'])
+
     def test_sub_factory(self):
         pass
 
